--[[ svn: $Id: InviteBlock.lua 42 2011-07-17 15:05:08Z crashmstr $

InviteBlock
    ...because they dont give up!

By crashmstr (wowzn@crashmstr.com)

Purpose:
This is an add-on to block incoming guild/party/trade/duel requests (except for friends)

Although I came up with the idea on my own, Ive borrowed a lot of the code from AntiSocial by Zariel / Kanne @ Trollbane.eu
Thank you!

(previously named ASSBlock)

--]]

InviteBlock = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local options = {
	type='group',
	args = {
		enabled = {
			type = 'toggle', name = 'Enable', desc = 'Toggles enabled status',
			get = 'IsEnabled', set = 'ToggleEnabled',
		},
		off = {
			type = 'toggle', name = 'Disable', desc = 'turns off',
			get = 'IsEnabled', set = 'TurnOff',
		},
		on = {
			type = 'toggle', name = 'Enable', desc = 'turns on',
			get = 'IsEnabled', set = 'TurnOn',
		},
		friends = {
			type = 'toggle', name = 'Allow Friends', desc = 'Toggles allowing friends',
			get = 'IsFriendsAllowed', set = 'ToggleFriendsAllowed',
		},
		party = {
			type = 'toggle', name = 'Block Party Invites', desc = 'Toggles party invite blocking',
			get = 'IsPartyBlocked', set = 'TogglePartyBlocked',
		},
		partyMessage = {
			type = 'text', name = 'Message for Blocked Party Invites',
			desc = 'The message to display in response to party invites',
			usage = "<your message here>",
			get = 'GetPartyMessage', set = 'SetPartyMessage',
		},
		trade = {
			type = 'toggle', name = 'Block Trade Invites', desc = 'Toggles trade blocking',
			get = 'IsTradeBlocked', set = 'ToggleTradeBlocked',
		},
		tradeMessage = {
			type = 'text', name = 'Message for Blocked Trades',
			desc = 'The message to display in response to trades',
			usage = "<your message here>",
			get = 'GetTradeMessage', set = 'SetTradeMessage',
		},
		duel = {
			type = 'toggle', name = 'Block Duel Invites', desc = 'Toggles duel invite blocking',
			get = 'IsDuelBlocked', set = 'ToggleDuelBlocked',
		},
		duelMessage = {
			type = 'text', name = 'Message for Blocked Duel Invites',
			desc = 'The message to display in response to duel invites',
			usage = "<your message here>",
			get = 'GetDuelMessage', set = 'SetDuelMessage',
		},
		guild = {
			type = 'toggle', name = 'Block Guild Invites', desc = 'Toggles guild invite blocking',
			get = 'IsGuildBlocked', set = 'ToggleGuildBlocked',
		},
		guildMessage = {
			type = 'text', name = 'Message for Blocked Guild Invites',
			desc = 'The message to display in response to guild invites',
			usage = "<your message here>",
			get = 'GetGuildMessage', set = 'SetGuildMessage',
		},
		reply = {
			type = 'toggle', name = 'Reply Message toggle', desc = 'Toggles the reply message',
			get = 'IsReply', set = 'ToggleReply',
		},
		followUp = {
			type = 'toggle', name = 'FollowUp Message toggle', desc = 'Toggles the followUp message',
			get = 'IsFollowUp', set = 'ToggleFollowUp',
		},
		followUpMessage = {
			type = 'text', name = 'FollowUp Message',
			desc = 'The message to display in addition to the reply message',
			usage = "<your message here>",
			get = 'GetFollowUpMessage', set = 'SetFollowUpMessage',
		},
		debug = {
			type = 'toggle', name = 'Debug window', desc = 'Toggles the debug window',
			get = 'IsDebug', set = 'ToggleDebug',
		},
	},
}

local stockMessage = "This is an automated reply to your request to duel/group/guild invite. I am using InviteBlock to decline since you are not in my guild, in my party, or on my friends list."
local defaults = {
	debug = true, enabled = true, reply = true, followUp = true,
	friendsAllowed = true, partyBlock = true,
	tradeBlock = false, duelBlock = true, guildBlock = true,
	message = {
		["GUILD_INVITE_REQUEST"] = stockMessage,
		["GUILD_REGISTRAR_SHOW"] = stockMessage,
		["PARTY_INVITE_REQUEST"] = stockMessage,
		["TRADE_SHOW"] = stockMessage,
		["DUEL_REQUESTED"] = stockMessage,
		["FOLLOW_UP"] = "If you think this is a mistake, please whisper back.",
	},
	blockLog = { },
}

InviteBlock:RegisterDB("InviteBlockDB", "InviteBlockDBPerChar")
InviteBlock:RegisterDefaults('profile', defaults)

function InviteBlock:IsEnabled() return self.db.profile.enabled; end
function InviteBlock:ToggleEnabled() self.db.profile.enabled = not self.db.profile.enabled; end
function InviteBlock:IsFriendsAllowed() return self.db.profile.friendsAllowed; end
function InviteBlock:ToggleFriendsAllowed() self.db.profile.friendsAllowed = not self.db.profile.friendsAllowed; end

function InviteBlock:TurnOn() self.db.profile.enabled = true; end
function InviteBlock:TurnOff() self.db.profile.enabled = false; end


function InviteBlock:IsPartyBlocked() return self.db.profile.partyBlock; end
function InviteBlock:TogglePartyBlocked() self.db.profile.partyBlock = not self.db.profile.partyBlock; end
function InviteBlock:GetPartyMessage() return self.db.profile.message["PARTY_INVITE_REQUEST"]; end
function InviteBlock:SetPartyMessage(msg) self.db.profile.message["PARTY_INVITE_REQUEST"] = msg; end

function InviteBlock:IsTradeBlocked() return self.db.profile.tradeBlock; end
function InviteBlock:ToggleTradeBlocked() self.db.profile.tradeBlock = not self.db.profile.tradeBlock; end
function InviteBlock:GetTradeMessage() return self.db.profile.message["TRADE_SHOW"]; end
function InviteBlock:SetTradeMessage(msg) self.db.profile.message["TRADE_SHOW"] = msg; end

function InviteBlock:IsDuelBlocked() return self.db.profile.duelBlock; end
function InviteBlock:ToggleDuelBlocked() self.db.profile.duelBlock = not self.db.profile.duelBlock; end
function InviteBlock:GetDuelMessage() return self.db.profile.message["DUEL_REQUESTED"]; end
function InviteBlock:SetDuelMessage(msg) self.db.profile.message["DUEL_REQUESTED"] = msg; end

function InviteBlock:IsGuildBlocked() return self.db.profile.guildBlock; end
function InviteBlock:ToggleGuildBlocked() self.db.profile.guildBlock = not self.db.profile.guildBlock; end
function InviteBlock:GetGuildMessage() return self.db.profile.message["GUILD_INVITE_REQUEST"]; end
function InviteBlock:SetGuildMessage(msg) self.db.profile.message["GUILD_INVITE_REQUEST"] = msg; end

function InviteBlock:IsReply() return self.db.profile.reply; end
function InviteBlock:ToggleReply() self.db.profile.reply = not self.db.profile.reply; end
function InviteBlock:IsFollowUp() return self.db.profile.followUp; end
function InviteBlock:ToggleFollowUp() self.db.profile.followUp = not self.db.profile.followUp; end
function InviteBlock:GetFollowUpMessage() return self.db.profile.message["FOLLOW_UP"]; end
function InviteBlock:SetFollowUpMessage(msg) self.db.profile.message["FOLLOW_UP"] = msg; end

function InviteBlock:IsDebug() return self.db.profile.debug; end
function InviteBlock:ToggleDebug() self.db.profile.debug = not self.db.profile.debug; end

function InviteBlock:OnInitialize()
    -- Called when the addon is loaded
end

function InviteBlock:OnEnable()
	-- Called when the addon is enabled
	self:RegisterEvent("GUILD_INVITE_REQUEST")
	self:RegisterEvent("GUILD_REGISTRAR_SHOW")
	self:RegisterEvent("PARTY_INVITE_REQUEST")
	self:RegisterEvent("TRADE_SHOW")
	self:RegisterEvent("TRADE_REQUEST")
	self:RegisterEvent("DUEL_REQUESTED")
	self:RegisterChatCommand("/inviteblock", "/invbl", options)
	self:Print("Anti-Social Software: Block invites (InviteBlock) ENABLED!")
end

function InviteBlock:OnDisable()
    -- Called when the addon is disabled
	self:Print("Anti-Social Software: Block invites (InviteBlock) DIS-ENABLED!")
end

function InviteBlock:IsFriend(requester)
	-- Check friend list firs, then guild, then party
	local isFriend = false
	if name and self.db.profile.friendsAllowed then
		for i=1,GetNumFriends() do
			local name = GetFriendInfo(i)
			--self:Print(name)
			if name == requester then
				isFriend = true
				break
			end
		end
		if not isFriend and IsInGuild() then
			for i = 1,GetNumGuildMembers(includeOffline) do
				local name = GetGuildRosterInfo(i)
				if name == requester then
					isFriend = true
					break
				end
			end
		end
		if not isFriend then
			--self:Print("Checking party members...")
			for i = 0,GetNumPartyMembers() do
				local name, realm
				name, realm = UnitName("party" .. i)
				--self:Print((name and name or "(nil)") .. (real and realm or "(nil)")  .. " ?==  " .. requester)
				if name == requester then
					isFriend = true
					break
				end
			end
		end
	end
	return isFriend
end

function InviteBlock:EnabledFor(type)
	local isEnabled = self.db.profile.enabled
	if isEnabled then
		if type == "TRADE_SHOW" then
			isEnabled = self.db.profile.tradeBlock
		elseif type == "DUEL_REQUESTED" then
			isEnabled = self.db.profile.duelBlock
		elseif type == "GUILD_INVITE_REQUEST" then
			isEnabled = self.db.profile.guildBlock
		elseif type == "PARTY_INVITE_REQUEST" then
			isEnabled = self.db.profile.partyBlock
		end
	end
	return isEnabled
end

function InviteBlock:SendReply(event, person)
	self:Print((self:IsReply() and "True" or "False") .. " " .. (event and event or "no event") .. " " .. (person and person or "no person"))
	if self:IsReply() and event and person then
		SendChatMessage(self.db.profile.message[event], "WHISPER", nil, person)
		if self:IsFollowUp() then
			SendChatMessage(self.db.profile.message["FOLLOW_UP"], "WHISPER", nil, person)
		end
	end
	self:LogBlock(event, person)
end

function InviteBlock:LogBlock(event, name)
	local index = 1 + table.getn(self.db.profile.blockLog)
	self.db.profile.blockLog[index] = {
		["date"] = date("%Y%m%d%H%M%S", time()),
		["event"] = event,
		["arg1"] = name,
		["player"] = UnitName('player'),
	}
end

function InviteBlock:GUILD_INVITE_REQUEST(name, guild)
	local event = "GUILD_INVITE_REQUEST"
	self:Print("We were invited to join guild " .. (guild and guild or "(nil)") .. " by "
				  .. (name and name or "(nil)") .. "! wowz!")
	if self:EnabledFor(event) then
		if not self:IsFriend(name) then
			self:Print(name .. " is not a friend, so send them a message and cancel the guild invite!")
			self:SendReply(event, name)
			HideUIPanel(GuildInviteFrame)
			DeclineGuild()
		end
	end
end

function InviteBlock:GUILD_REGISTRAR_SHOW(arg1, arg2)
	local event = "GUILD_REGISTRAR_SHOW"
	self:Print("We saw a guild registrar?!?"
				  .. (arg2 and arg2 or "nil") 
					  .. (arg1 and arg1 or "nil")
					  .. "! wowz!")
end

function InviteBlock:DUEL_REQUESTED(name)
	local event = "DUEL_REQUESTED"
	self:Print("We were invited to duel with " .. (name and name or "(nil)"))
	if self:EnabledFor(event) then
		if not self:IsFriend(name) then
			self:Print(name .. " is not a friend, so send them a message and cancel the duel invite!")
			self:SendReply(event, name)
			HideUIPanel(StaticPopup1)
			CancelDuel()
		end
	end
end

function InviteBlock:PARTY_INVITE_REQUEST(name)
	local event = "PARTY_INVITE_REQUEST"
	self:Print("We were invited to party up by " .. (name and name or "(nil)"))
	if self:EnabledFor(event) then
		if not self:IsFriend(name) then
			self:Print(name .. " is not a friend, so send them a message and cancel the party invite!")
			self:SendReply(event, name)
			HideUIPanel(StaticPopup1)
			DeclineGroup()
		end
	end
end

function InviteBlock:TRADE_SHOW(name)
	local event = "TRADE_SHOW"
	--this event does not get a parameter, so we cannot check the sender!?!?!?
	self:Print("We were invited to trade..." .. (name and name or "(nil)"))
	if self:EnabledFor(event) then
		--if not self:IsFriend(name) then
			--self:Print(name .. " is not a friend, so send them a message and cancel the trade!")
			--self:SendReply(event, name)
			HideUIPanel(StaticPopup1)
			CancelTrade()
		--end
	end
end

function InviteBlock:TRADE_REQUEST(name)
	--this event does not fire
	self:Print("TRADE_REQUEST event..." .. (name and name or "(nil)"))
end
